using System.Threading;
using MediatR;
using Todo.Core.Application.Notifications.Todo;
using Todo.Core.Interfaces.Persistence;
using Todo.Core.Interfaces.User;

namespace Todo.Core.Application.Commands.Todo;

public record ReplaceTodoCommand(
    string Id,
    string Title,
    string? Project,
    string? Description,
    bool Status,
    long Created,
    string? Priority) : IRequest<Unit>
{
    internal class Handler : IRequestHandler<ReplaceTodoCommand, Unit>
    {
        private readonly ICurrentUserService _currentUserService;
        private readonly ITodoRepository _todoRepository;
        private readonly IMediator _mediator;

        public Handler(
            ICurrentUserService currentUserService,
            ITodoRepository todoRepository,
            IMediator mediator)
        {
            _currentUserService = currentUserService;
            _todoRepository = todoRepository;
            _mediator = mediator;
        }

        public async Task<Unit> Handle(ReplaceTodoCommand request, CancellationToken cancellationToken)
        {
            var userId = _currentUserService.GetUserId();
            if (userId is null)
                throw new InvalidOperationException("User was not found");
            //TODO: Make sure the author actually owns the todo 
            await _todoRepository.UpdateTodoAsync(request.To(userId));

            await _mediator.Publish(new TodoUpdated(request.Id), cancellationToken);

            return Unit.Value;
        }
    }

    private Entities.Todo To(string authorId) => new(
        Id,
        Title,
        Status,
        Project,
        authorId,
        Description,
        Created,
        Priority);
}